package org.example.project.ui.Screens

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.BoxWithConstraints
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentWidth
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.Button
import androidx.compose.material.ButtonDefaults
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Scaffold
import androidx.compose.material.Text
import androidx.compose.material.TextField
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.TextUnit
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import cafe.adriel.voyager.core.screen.Screen
import cafe.adriel.voyager.navigator.LocalNavigator
import cafe.adriel.voyager.navigator.currentOrThrow
import org.example.project.ViewModel.TimerViewModel

class RegRunnerScreen(private val timerViewModel: TimerViewModel) : Screen {

    @Composable
    override fun Content() {
        val navigator = LocalNavigator.currentOrThrow

        Scaffold(
            topBar = { RegRunnerTopBar() },
            bottomBar = { RegRunnerBottomBar(timerViewModel = timerViewModel) }
        ) { paddingValues ->
            Box(
                modifier = Modifier.padding(paddingValues)
            ) {
                BoxWithConstraints(
                    modifier = Modifier
                        .fillMaxWidth()
                        .align(Alignment.Center)
                        .padding(horizontal = 100.dp)
                ) {
                    val fontSize = (maxWidth.value / 25).sp
                    val fontSizeFirst = (maxWidth.value / 20).sp
                    val fontSizeSecond = (maxWidth.value / 35).sp
                    val labelWidth = 120.dp
                    var email by remember { mutableStateOf("") }
                    var password by remember { mutableStateOf("") }

                    Column(
                        modifier = Modifier
                            .fillMaxWidth(),
                        horizontalAlignment = Alignment.CenterHorizontally,
                    ) {
                        Spacer(Modifier.weight(0.25f))

                        Text(
                            text = "Регистрация бегуна",
                            color = Color.DarkGray,
                            textAlign = TextAlign.Center,
                            fontSize = fontSizeFirst,
                            modifier = Modifier.fillMaxWidth()
                        )

                        Spacer(Modifier.weight(0.25f))

                        Text(
                            text = "Пожалуйста, заполните всю информацию, чтобы зарегестрироваться в качестве участника",
                            color = Color.DarkGray,
                            textAlign = TextAlign.Center,
                            fontSize = fontSizeSecond,
                            modifier = Modifier.fillMaxWidth(),
                        )

                        Spacer(Modifier.weight(0.2f))

                        Row(
                            modifier = Modifier.fillMaxWidth(),
                            horizontalArrangement = Arrangement.spacedBy(32.dp)
                        ) {
                            Column(modifier = Modifier.weight(1f)) {
                                FieldWithLabel(
                                    label = "Email:",
                                    value = email,
                                    onValueChange = { email = it },
                                    placeholder = "Enter your email address",
                                    fontSize = fontSize,
                                    labelWidth = labelWidth
                                )

                                Spacer(modifier = Modifier.weight(0.1f))

                                FieldWithLabel(
                                    label = "Password:",
                                    value = password,
                                    onValueChange = { password = it },
                                    placeholder = "Enter your password",
                                    fontSize = fontSize,
                                    labelWidth = labelWidth
                                )
                            }

                            Column(modifier = Modifier.weight(1f)) {
                                FieldWithLabel(
                                    label = "Email:",
                                    value = email,
                                    onValueChange = { email = it },
                                    placeholder = "Enter your email address",
                                    fontSize = fontSize,
                                    labelWidth = labelWidth
                                )

                                Spacer(modifier = Modifier.weight(0.1f))

                                FieldWithLabel(
                                    label = "Password:",
                                    value = password,
                                    onValueChange = { password = it },
                                    placeholder = "Enter your password",
                                    fontSize = fontSize,
                                    labelWidth = labelWidth
                                )
                            }
                        }
                    }
                }
            }
        }
    }

    @Composable
    fun FieldWithLabel(
        label: String,
        value: String,
        onValueChange: (String) -> Unit,
        placeholder: String,
        fontSize: TextUnit,
        labelWidth: Dp,
        enabled: Boolean = true
    ) {
        Row(
            verticalAlignment = Alignment.CenterVertically,
            modifier = Modifier.fillMaxWidth()
        ) {
            Box(modifier = Modifier.width(labelWidth)) {
                Text(
                    text = label,
                    color = Color.DarkGray,
                    textAlign = TextAlign.Start,
                    fontSize = fontSize,
                    modifier = Modifier.padding(start = 40.dp),
                )
            }

            TextField(
                value = value,
                onValueChange = onValueChange,
                placeholder = {
                    if (placeholder.isNotEmpty()) {
                        Text(
                            text = placeholder,
                            fontSize = fontSize,
                            color = Color.LightGray
                        )
                    }
                },
                textStyle = androidx.compose.ui.text.TextStyle(fontSize = fontSize),
                modifier = Modifier.weight(1f),
                enabled = enabled
            )
        }
    }

    @Composable
    private fun RegRunnerTopBar() {
        val navigator = LocalNavigator.currentOrThrow

        Box {
            BoxWithConstraints(
                modifier = Modifier
                    .fillMaxWidth()
                    .align(Alignment.Center)
            ) {
                val fontSizeButton = (maxWidth.value / 50).sp
                val fontSizeText = (maxWidth.value / 25).sp

                Row(
                    modifier = Modifier
                        .fillMaxWidth()
                        .background(Color.DarkGray)
                        .padding(16.dp),
                    verticalAlignment = Alignment.CenterVertically,
                    horizontalArrangement = Arrangement.SpaceBetween
                ) {

                    Button(
                        onClick = { navigator.push(MainScreen()) },
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = Color.LightGray,
                            contentColor = Color.Black
                        ),
                        elevation = null,
                        shape = RoundedCornerShape(15.dp),
                        modifier = Modifier
                            .wrapContentWidth()

                    ) {
                        Text(
                            text = "Назад",
                            color = Color.Black,
                            fontSize = fontSizeButton
                        )
                    }

                    Spacer(modifier = Modifier.width(24.dp))

                    Text(
                        text = "MARATHON SKILLS 2016",
                        style = MaterialTheme.typography.h6,
                        modifier = Modifier.weight(1f),
                        textAlign = TextAlign.Left,
                        fontWeight = FontWeight.Bold,
                        fontSize = fontSizeText,
                        color = Color.White
                    )

                    Spacer(modifier = Modifier.width(64.dp))
                }
            }
        }
    }

    @Composable
    private fun RegRunnerBottomBar(timerViewModel: TimerViewModel) {
        val remainingTime = timerViewModel.remainingTime.collectAsState().value
        Box {
            BoxWithConstraints(
                modifier = Modifier
                    .align(Alignment.Center)
            ) {
                val fontSize = (maxWidth.value / 50).sp

                Column(
                    modifier = Modifier
                        .fillMaxWidth()
                        .background(Color.DarkGray)
                        .padding(8.dp),
                    horizontalAlignment = Alignment.CenterHorizontally
                ) {
                    Text(
                        text = remainingTime,
                        color = Color.White,
                        fontSize = fontSize
                    )
                }
            }
        }
    }
}